/*************************************************
* Enumerations Header File                       *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_ENUMS_H__
#define BOTAN_ENUMS_H__

#include <botan/config.h>

namespace Botan {

/*************************************************
* ASN.1 Type and Class Tags                      *
*************************************************/
enum ASN1_Tag {
   NO_OBJECT        = 0x00,
   BOOLEAN          = 0x01,
   INTEGER          = 0x02,
   BIT_STRING       = 0x03,
   OCTET_STRING     = 0x04,
   NULL_TAG         = 0x05,
   OBJECT_ID        = 0x06,
   SEQUENCE         = 0x10,
   SET              = 0x11,

   NUMERIC_STRING   = 0x12,
   PRINTABLE_STRING = 0x13,
   IA5_STRING       = 0x16,
   VISIBLE_STRING   = 0x1A,

   UTC_TIME         = 0x17,
   GENERALIZED_TIME = 0x18,

   CONSTRUCTED      = 0x20,

   UNIVERSAL        = 0x00,
   APPLICATION      = 0x40,
   CONTEXT_SPECIFIC = 0x80,
   PRIVATE          = 0xC0
};

/*************************************************
* X.509v3 Key Constraints                        *
*************************************************/
enum Key_Constraints {
   DIGITAL_SIGNATURE  = 32768,
   NON_REPUDATION     = 16384,
   KEY_ENCIPHERMENT   = 8192,
   DATA_ENCIPHERMENT  = 4096,
   KEY_AGREEMENT      = 2048,
   KEY_CERT_SIGN      = 1024,
   CRL_SIGN           = 512,
   ENCIPHER_ONLY      = 256,
   DECIPHER_ONLY      = 128
};

/*************************************************
* Various Other Enumerations                     *
*************************************************/
enum RNG_Quality { Nonce = 0, PublicValue = 0, SessionKey, LongTermKey };

enum X509_Encoding { RAW_BER, PEM };

enum Cipher_Dir { ENCRYPTION, DECRYPTION };

enum Signature_Format { IEEE_1363, DER_SEQUENCE };

}

#endif
