/*************************************************
* Number Theory Header File                      *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_NUMBTHRY_H__
#define BOTAN_NUMBTHRY_H__

#include <botan/bigint.h>
#include <botan/reducer.h>

namespace Botan {

/*************************************************
* Fused Arithmetic Operations                    *
*************************************************/
BigInt mul_add(const BigInt&, const BigInt&, const BigInt&);
BigInt sub_mul(const BigInt&, const BigInt&, const BigInt&);

/*************************************************
* Number Theory Functions                        *
*************************************************/
inline BigInt abs(const BigInt& n) { return n.abs(); }

void divide(const BigInt&, const BigInt&, BigInt&, BigInt&);
void positive_divide(const BigInt&, const BigInt&, BigInt&, BigInt&);
void modifying_divide(BigInt&, BigInt&, BigInt&);

BigInt gcd(const BigInt&, const BigInt&);
BigInt lcm(const BigInt&, const BigInt&);

BigInt square(const BigInt&);
BigInt inverse_mod(const BigInt&, const BigInt&);
s32bit jacobi(const BigInt&, const BigInt&);
BigInt power(const BigInt&, u32bit);

/*************************************************
* Modular Exponentiation                         *
*************************************************/
BigInt power_mod(const BigInt&, const BigInt&, ModularReducer*);
BigInt power_mod(const BigInt&, const BigInt&, const BigInt&);

/*************************************************
* Primality Testing                              *
*************************************************/
bool check_prime(const BigInt&);
bool is_prime(const BigInt&);
bool verify_prime(const BigInt&);

s32bit simple_primality_tests(const BigInt&);
bool passes_mr_tests(const BigInt&, u32bit = 1);
bool run_primality_tests(const BigInt&, u32bit = 1);

/*************************************************
* Random Number Generation                       *
*************************************************/
BigInt random_integer(u32bit, RNG_Quality);
BigInt random_integer(const BigInt&, const BigInt&, RNG_Quality);
BigInt random_prime(u32bit, RNG_Quality, const BigInt& = 1,
                    u32bit = 1, u32bit = 2);
BigInt random_safe_prime(u32bit, RNG_Quality);

SecureVector<byte> generate_dsa_primes(BigInt&, BigInt&, u32bit);
bool generate_dsa_primes(BigInt&, BigInt&, const byte[], u32bit, u32bit,
                         u32bit = 0);

/*************************************************
* Various Utility Functions                      *
*************************************************/
u32bit low_zero_bits(const BigInt&);

}

#endif
