/*************************************************
* X.509 Certificate Authority Header File        *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_X509_CA_H__
#define BOTAN_X509_CA_H__

#include <botan/x509cert.h>
#include <botan/x509opt.h>
#include <botan/pkcs8.h>
#include <botan/pubkey.h>

namespace Botan {

/*************************************************
* X.509 Certificate Authority                    *
*************************************************/
class X509_CA
   {
   public:
      X509_Certificate sign_key(const X509_PublicKey&,
                                const X509_Cert_Options&) const;

      X509_Certificate ca_certificate() const;

      X509_CA(const X509_Certificate&, const PKCS8_PrivateKey&);
      X509_CA(const PKCS8_PrivateKey&, X509_Cert_Options&);
      ~X509_CA();
   private:
      X509_CA(const X509_CA&) {}
      X509_CA& operator=(const X509_CA&) { return (*this); }
      X509_Certificate make_cert(const SecureVector<byte>&,
                                 const X509_Cert_Options&,
                                 Key_Constraints) const;
      Key_Constraints find_constraints(const X509_PublicKey&,
                                       Key_Constraints) const;
      void do_ext(DER_Encoder&, DER_Encoder&, const std::string&, bool) const;
      AlgorithmIdentifier ca_sig_algo;
      std::map<std::string, std::string> ca_dn_info;
      X509_Certificate cert;
      PK_Signer* signer;
   };

}

#endif
