/*************************************************
* X.509 Certificate Store Header File            *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#ifndef BOTAN_X509_CERT_STORE_H__
#define BOTAN_X509_CERT_STORE_H__

#include <botan/x509cert.h>

namespace Botan {

/*************************************************
* X.509 Certificate Store                        *
*************************************************/
class X509_Store
   {
   public:
      bool valid_cert(const X509_Certificate&) const;

      std::vector<X509_Certificate> find(const std::string&,
                                         const std::string&) const;

      std::string PEM_encode() const;

      void add_cert(const X509_Certificate&, bool = false);
      void add_certs(DataSource&, X509_Encoding = PEM);
      void add_trusted_certs(DataSource&, X509_Encoding = PEM);
   private:
      void do_add_certs(DataSource&, X509_Encoding, bool);
      u32bit find_parent_of(const X509_Certificate&) const;
      bool check_sig(const X509_Certificate&, const X509_Certificate&) const;
      static const u32bit NO_CERT_FOUND = 0xFFFFFFFF;
      std::vector<X509_Certificate> certs;
   };

}

#endif
