/*************************************************
* Memory Mapping Allocator Header File           *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/manalloc.h>
#include <utility>
#include <vector>
#include <map>

#ifndef BOTAN_EXT_MMAP_ALLOCATOR_H__
#define BOTAN_EXT_MMAP_ALLOCATOR_H__

namespace Botan {

/*************************************************
* Memory Mapping Allocator                       *
*************************************************/
class MemoryMapping_Allocator : public ManagedAllocator
   {
   public:
      SecureAllocator* clone() { return this; }

      MemoryMapping_Allocator(const std::string& = "");
      ~MemoryMapping_Allocator();
   private:
      struct Mapped_Buffer { void* buf; u32bit length; int fd; };

      void* alloc_block(u32bit) const;
      void dealloc_block(void*, u32bit) const;
      std::string temp_dir(const std::string&) const;
      void* mmap_block(u32bit) const;
      void munmap_block(void*, u32bit, int) const;

      const std::string path;
      mutable std::map<void*, std::pair<int, u32bit> > allocated;
      mutable std::vector<Mapped_Buffer> cache;
   };

}

#endif
