/*************************************************
* BeOS EntropySource Source File                 *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/es_beos.h>
#include <botan/util.h>
#include <kernel/OS.h>

namespace Botan {

/*************************************************
* BeOS_EntropySource Constructor                 *
*************************************************/
BeOS_EntropySource::BeOS_EntropySource()
   {
   write_pos = read_pos = 0;
   }

/*************************************************
* Fast Poll                                      *
*************************************************/
u32bit BeOS_EntropySource::fast_poll(byte out[], u32bit length)
   {
   if(read_pos == buffer.size())
      slow_poll(out, length);

   system_info sys_info;
   get_system_info(&sys_info);
   add_bytes(&sys_info, sizeof(system_info));

   return copy_out(out, length, 16);
   }

/*************************************************
* Slow Poll                                      *
*************************************************/
u32bit BeOS_EntropySource::slow_poll(byte out[], u32bit length)
   {
   team_info info_team;
   int32 team_cookie = 0;
   while(get_next_team_info(&team_cookie, &info_team) == B_OK)
      {
      add_bytes(&info_team, sizeof(team_info));

      thread_info info_thr;
      int32 thr_cookie = 0;
      team_id id = info_team.team;

      while(get_next_thread_info(id, &thr_cookie, &info_thr) == B_OK)
         add_bytes(&info_thr, sizeof(thread_info));
      }

   return copy_out(out, length, 128);
   }

/*************************************************
* Add entropy to the internal buffer             *
*************************************************/
void BeOS_EntropySource::add_bytes(const void* entropy_ptr, u32bit length)
   {
   const byte* bytes = (const byte*)entropy_ptr;
   while(length)
      {
      u32bit copied = std::min(length, buffer.size() - write_pos);
      xor_buf(buffer + write_pos, bytes, copied);
      bytes += copied;
      length -= copied;
      write_pos = (write_pos + copied) % buffer.size();
      }
   }

/*************************************************
* Add entropy to the internal buffer             *
*************************************************/
u32bit BeOS_EntropySource::copy_out(byte out[], u32bit length,
                                    u32bit max_read)
   {
   length = std::min(length, max_read);
   u32bit copied = std::min(length, buffer.size() - read_pos);
   xor_buf(out, buffer + read_pos, copied);
   read_pos = (read_pos + copied) % buffer.size();
   return copied;
   }

}
