/*************************************************
* Fused Arithmetic Operations Source File        *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/numthry.h>
#include <botan/mp_core.h>

namespace Botan {

/*************************************************
* Fused Multiply/Add Operation                   *
*************************************************/
BigInt mul_add(const BigInt& a, const BigInt& b, const BigInt& c)
   {
   if(c.is_negative())
      throw Invalid_Argument("mul_add: Third argument must be positive");

   BigInt::Sign sign = BigInt::Positive;
   if(a.sign() != b.sign())
      sign = BigInt::Negative;

   const u32bit a_sw = a.sig_words();
   const u32bit b_sw = b.sig_words();
   const u32bit c_sw = c.sig_words();

   BigInt z(sign, std::max(a.size() + b.size(), c_sw) + 1);
   bigint_mul3(z.get_reg(), z.size(),
               a.data(), a.size(), a_sw,
               b.data(), b.size(), b_sw);
   const u32bit z_size = std::max(z.sig_words(), c_sw);
   bigint_add2(z.get_reg(), z_size, c.data(), c_sw);
   return z;
   }

/*************************************************
* Fused Subtract/Multiply Operation              *
*************************************************/
BigInt sub_mul(const BigInt& a, const BigInt& b, const BigInt& c)
   {
   if(a.is_negative() || b.is_negative())
      throw Invalid_Argument("sub_mul: First two arguments must be positive");

   BigInt x = a;
   x -= b;
   x *= c;
   return x;
   }

}
