/*************************************************
* MP Simple Multiplication Source File           *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/mp_core.h>
#include <botan/mp_mul.h>

extern "C" {

/*************************************************
* Simple O(N^2) Multiplication                   *
*************************************************/
void bigint_simple_mul(word* const z,
                       const word* x, u32bit x_size,
                       const word* y, u32bit y_size)
   {
   const u32bit blocks = y_size - (y_size % 8);

   for(u32bit j = 0; j != x_size; j++)
      {
      const word x_j = x[j];

      word carry = 0;
      for(u32bit k = 0; k != blocks; k += 8)
         {
         dword product;

         product = (dword)z[j+k+0] + (dword)x_j * y[k+0] + carry;
         z[j+k+0] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+1] + (dword)x_j * y[k+1] + carry;
         z[j+k+1] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+2] + (dword)x_j * y[k+2] + carry;
         z[j+k+2] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+3] + (dword)x_j * y[k+3] + carry;
         z[j+k+3] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+4] + (dword)x_j * y[k+4] + carry;
         z[j+k+4] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+5] + (dword)x_j * y[k+5] + carry;
         z[j+k+5] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+6] + (dword)x_j * y[k+6] + carry;
         z[j+k+6] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);

         product = (dword)z[j+k+7] + (dword)x_j * y[k+7] + carry;
         z[j+k+7] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);
         }

      for(u32bit k = blocks; k != y_size; k++)
         {
         dword product = (dword)z[j+k] + (dword)x_j * y[k] + carry;
         z[j+k] = MP_LOW_WORD(product);
         carry = MP_HIGH_WORD(product);
         }
      z[j+y_size] = carry;
      }
   }

}
