/*************************************************
* Modular Reducer Source File                    *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/reducer.h>

namespace Botan {

/*************************************************
* Construct a ModularReducer                     *
*************************************************/
ModularReducer::ModularReducer(const BigInt& n) : modulus(n)
   {
   if(modulus < 0)
      throw Invalid_Argument("ModularReducer: modulus must be positive");
   if(modulus.size() > 8 && !power_of_2(modulus.size()))
      modulus.grow_reg((1 << high_bit(modulus.size())) - modulus.size());
   }

/*************************************************
* Modular Multiplication                         *
*************************************************/
BigInt ModularReducer::multiply(const BigInt& a, const BigInt& b) const
   {
   return reduce(a * b);
   }

/*************************************************
* Modular Squaring                               *
*************************************************/
BigInt ModularReducer::square(const BigInt& a) const
   {
   return reduce(a * a);
   }

/*************************************************
* Convert an integer into the modular form       *
*************************************************/
BigInt ModularReducer::convert_to(const BigInt& n) const
   {
   return reduce(n);
   }

/*************************************************
* Convert an integer from the modular form       *
*************************************************/
BigInt ModularReducer::convert_from(const BigInt& n) const
   {
   return n;
   }

}
