/*************************************************
* SEAL Gamma Function Source File                *
* (C) 1999-2002 The Botan Project                *
*************************************************/

#include <botan/seal.h>
#include <botan/sha160.h>

namespace Botan {

/*************************************************
* Gamma Constructor                              *
*************************************************/
Gamma::Gamma(const byte UK[20])
   {
   for(u32bit j = 0; j != 5; j++)
      K[j] = make_u32bit(UK[4*j], UK[4*j+1], UK[4*j+2], UK[4*j+3]);
   last_index = 0xFFFFFFFF;
   }

/*************************************************
* Gamma Function                                 *
*************************************************/
u32bit Gamma::operator()(u32bit index)
   {
   u32bit new_index = index / 5;
   if(new_index != last_index)
      {
      SecureBuffer<byte, 80> W;
      for(u32bit j = 0; j != 4; j++)
         W[j] = get_byte(j, new_index);
      SHA_160 sha;
      sha.digest = K;
      sha.hash(W);
      digest = sha.digest;
      last_index = new_index;
      }
   return digest[index % 5];
   }

}
